package ink.wgink.module.instantmessage.websocket.handler.text;

import com.alibaba.fastjson.JSONException;
import com.alibaba.fastjson.JSONObject;
import ink.wgink.module.instantmessage.websocket.enums.MessageTypeEnum;
import ink.wgink.module.instantmessage.websocket.enums.StatusEnum;
import ink.wgink.module.instantmessage.websocket.exception.*;
import ink.wgink.module.instantmessage.websocket.manager.WebSocketChannelManager;
import ink.wgink.module.instantmessage.websocket.pojo.WebSocketClientMessage;
import ink.wgink.module.instantmessage.websocket.pojo.WebSocketSession;
import ink.wgink.module.instantmessage.websocket.pojo.body.IdsBody;
import ink.wgink.module.instantmessage.websocket.pojo.body.RegisterBody;
import ink.wgink.module.instantmessage.websocket.pojo.body.StatusBody;
import io.netty.channel.Channel;
import io.netty.channel.ChannelHandlerContext;
import io.netty.handler.codec.http.websocketx.TextWebSocketFrame;
import org.apache.commons.compress.utils.Sets;
import org.apache.commons.lang3.StringUtils;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

/**
 * @ClassName: WebSocketTextHandler
 * @Description: WebSocket文本处理器
 * @Author: wanggeng
 * @Date: 2021/9/12 8:08 下午
 * @Version: 1.0
 */
public class WebSocketTextHandler {

    public void handler(ChannelHandlerContext ctx, TextWebSocketFrame textWebSocketFrame) {
        WebSocketClientMessage clientSocketMessage = null;
        StatusBody statusBody = null;
        try {
            clientSocketMessage = JSONObject.parseObject(textWebSocketFrame.text(), WebSocketClientMessage.class);
            if (clientSocketMessage.getType() == null) {
                throw new TypeException("Type类型不能为空");
            }
            if (StringUtils.isBlank(clientSocketMessage.getBody())) {
                throw new BodyException("Body主体不能为空");
            }
            if (MessageTypeEnum.REGISTER.getValue() == clientSocketMessage.getType()) {
                // 客户端注册消息
                statusBody = clientRegisterSession(ctx.channel(), clientSocketMessage);
            } else if (MessageTypeEnum.MESSAGE.getValue() == clientSocketMessage.getType()) {
                // 文本消息
                sendText(ctx.channel(), clientSocketMessage);
            } else if (MessageTypeEnum.MESSAGE_GROUP.getValue() == clientSocketMessage.getType()) {
                // 群发消息
                sendGroupText(ctx.channel(), clientSocketMessage);
            } else if (MessageTypeEnum.SEARCH_ONLINE_USER.getValue() == clientSocketMessage.getType()) {
                // 用户在线状态
                listOnlineUser(ctx.channel(), clientSocketMessage);
            } else if (MessageTypeEnum.STATUS_RECEIVE.getValue() == clientSocketMessage.getType()) {
                // 消息接收状态
                sendReceiveStatus(clientSocketMessage);
            } else {
                throw new TypeException("请求类型错误");
            }
        } catch (SessionException e) {
            // 没有登录时，返回内容，关闭连接
            statusBody = new StatusBody(StatusEnum.SESSION_ERROR.getValue(), StatusEnum.SESSION_ERROR, e.getMessage());
            clientSocketMessage.setBody(JSONObject.toJSONString(statusBody));
            WebSocketChannelManager.getInstance().sendText(ctx.channel(), clientSocketMessage);
            ctx.close();
        } catch (TypeException e) {
            statusBody = new StatusBody(StatusEnum.TYPE_ERROR.getValue(), StatusEnum.SESSION_ERROR, e.getMessage());
        } catch (BodyException e) {
            statusBody = new StatusBody(StatusEnum.BODY_ERROR.getValue(), StatusEnum.SESSION_ERROR, e.getMessage());
        } catch (UserException e) {
            statusBody = new StatusBody(StatusEnum.SESSION_ERROR.getValue(), StatusEnum.SESSION_ERROR, e.getMessage());
        } catch (JSONException e) {
            clientSocketMessage = new WebSocketClientMessage();
            clientSocketMessage.setSystem(true);
            clientSocketMessage.setType(MessageTypeEnum.MESSAGE_SYSTEM.getValue());
            clientSocketMessage.setFrom(WebSocketChannelManager.FORM_SYSTEM);
            statusBody = new StatusBody(StatusEnum.TO_ERROR.getValue(), StatusEnum.MESSAGE_ERROR, e.getMessage());
        } finally {
            if (statusBody != null && ctx.channel().isOpen()) {
                clientSocketMessage.setBody(JSONObject.toJSONString(statusBody));
                WebSocketChannelManager.getInstance().sendText(ctx.channel(), clientSocketMessage);
            }
        }
    }

    /**
     * 发送接收状态
     *
     * @param clientSocketMessage 客户端消息
     */
    private void sendReceiveStatus(WebSocketClientMessage webSocketClientMessage) {
        List<String> toUserIds = new ArrayList<>(Sets.newHashSet(webSocketClientMessage.getTo().split(",")));
        List<WebSocketSession> webSocketSessions = WebSocketChannelManager.getInstance().listOnlineUser(toUserIds);
        for (WebSocketSession webSocketSession : webSocketSessions) {
            WebSocketChannelManager.getInstance().sendText(webSocketSession.getChannel(), webSocketClientMessage);
        }
    }

    /**
     * 客户端会话注册
     *
     * @param channel             会话通道
     * @param clientSocketMessage 客户端信息
     * @return
     * @throws SessionException
     */
    private StatusBody clientRegisterSession(Channel channel, WebSocketClientMessage clientSocketMessage) throws SessionException {
        RegisterBody registerBody = JSONObject.parseObject(clientSocketMessage.getBody(), RegisterBody.class);
        // 更新通道
        WebSocketChannelManager.getInstance().addChannel(registerBody.getSessionId(), clientSocketMessage.getFrom(), channel);
        return new StatusBody(StatusEnum.SUCCESS.getValue(), StatusEnum.SUCCESS, "OK");
    }

    /**
     * 发送文字消息
     *
     * @param channel                通道
     * @param webSocketClientMessage 客户端消息
     * @throws UserException
     */
    private void sendText(Channel channel, WebSocketClientMessage webSocketClientMessage) throws UserException {
        if (StringUtils.isBlank(webSocketClientMessage.getTo())) {
            throw new UserException("To 值不能为空");
        }
        webSocketClientMessage.setSystem(false);
        List<WebSocketSession> webSocketSessions = WebSocketChannelManager.getInstance().listOnlineUser(webSocketClientMessage.getTo());
        // 返回失败结果
        if (webSocketSessions.isEmpty()) {
            sendTextFailed(channel, webSocketClientMessage, "用户不在线");
            return;
        }
        // 发送消息
        for (WebSocketSession webSocketSession : webSocketSessions) {
            WebSocketChannelManager.getInstance().sendText(webSocketSession.getChannel(), webSocketClientMessage);
        }
    }

    /**
     * 群发文字消息
     *
     * @param fromChannel            发送通道
     * @param webSocketClientMessage 客户端消息
     * @throws UserException
     */
    private void sendGroupText(Channel fromChannel, WebSocketClientMessage webSocketClientMessage) throws UserException {
        if (StringUtils.isBlank(webSocketClientMessage.getTo())) {
            throw new UserException("To 值不能为空");
        }
        webSocketClientMessage.setSystem(false);
        List<String> toUserIds = new ArrayList<>(Sets.newHashSet(webSocketClientMessage.getTo().split(",")));
        List<WebSocketSession> webSocketSessions = WebSocketChannelManager.getInstance().listOnlineUser(toUserIds);
        // 返回失败结果
        if (webSocketSessions.isEmpty()) {
            sendTextFailed(fromChannel, webSocketClientMessage, "无用户在线");
            return;
        }
        // 发送消息
        for (WebSocketSession webSocketSession : webSocketSessions) {
            WebSocketChannelManager.getInstance().sendText(webSocketSession.getChannel(), webSocketClientMessage);
        }
    }

    /**
     * 在线用户列表
     *
     * @param fromChannel            通道
     * @param webSocketClientMessage 消息体
     * @throws UserException
     */
    private void listOnlineUser(Channel fromChannel, WebSocketClientMessage webSocketClientMessage) throws UserException {
        webSocketClientMessage.setSystem(true);
        webSocketClientMessage.setTo(webSocketClientMessage.getFrom());
        IdsBody idsBody = JSONObject.parseObject(webSocketClientMessage.getBody(), IdsBody.class);
        if (idsBody.getIds() == null || idsBody.getIds().isEmpty()) {
            sendTextFailed(fromChannel, webSocketClientMessage, "用户ID列表为空");
            return;
        }
        List<WebSocketSession> webSocketSessions = WebSocketChannelManager.getInstance().listOnlineUser(idsBody.getIds());
        Set<String> idSets = new HashSet<>();
        for (WebSocketSession webSocketSession : webSocketSessions) {
            idSets.add(webSocketSession.getUserId());
        }
        idsBody = new IdsBody();
        idsBody.setIds(new ArrayList<>(idSets));
        webSocketClientMessage.setType(MessageTypeEnum.STATUS_SEND_ONLINE.getValue());
        webSocketClientMessage.setBody(JSONObject.toJSONString(idsBody));
        sendText(fromChannel, webSocketClientMessage);
    }

    /**
     * 发送失败内容
     *
     * @param fromChannel            来源通道
     * @param webSocketClientMessage webSocket客户端消息
     * @param msg
     */
    public static void sendTextFailed(Channel fromChannel, WebSocketClientMessage webSocketClientMessage, String msg) {
        webSocketClientMessage.setType(MessageTypeEnum.STATUS_SEND.getValue());
        webSocketClientMessage.setBody(JSONObject.toJSONString(new StatusBody(StatusEnum.FAILED.getValue(), StatusEnum.FAILED, msg)));
        WebSocketChannelManager.getInstance().sendText(fromChannel, webSocketClientMessage);
    }

}
